﻿# Name: gcp_poc_setup.ps1
# Purpose: Automate deployment of Velostrata Infra Deplyment in GCP.
# Created: 09/11/2017
# Author: Alon Pildus , Boaz Harel
# Created For: velostrata.com
# Version: 1.0

# Create GCP Deployment
# Create VPC & Subnets
# Create FW Tags	
# Create Routing Table
# Deploy VyOS 1.8 
# Create GCP Service Account
# Create Velostrata GCP Custom Role for Operations

write-host ("`nVelostrata PoC Google Cloud Setup Script") -foreground "Green"


################## Login and Subscription ###################

# Sign-in with GCP Account Credentials

$GCPAccount = Read-Host "`nEnter an administrative Google Account - JDOE@GMAIL.COM"

$GCPProject = Read-Host "`nEnter an existing Google Cloud PROJECT ID to be used for the Velostrata PoC Setup"

gcloud auth login $GCPAccount --no-launch-browser --brief

gcloud config set project $GCPProject

################## Environment Variables ###################


$VerbosePreference = "continue"

Write-Host

Write-verbose "`nSetting Environment Variables"

$GCPLocation = @("us-central1", "europe-west1", "us-west1", "asia-east1", "asia-south1", "us-east1", "asia-northeast1", "asia-southeast1", "us-east4", "australia-southeast1", "europe-west2", "europe-west3", "southamerica-east1")
$GCPLocationToZones = @{
"asia-east1" = "asia-east1-a", "asia-east1-b", "asia-east1-c" ;
"asia-northeast1" = "asia-northeast1-c", "asia-northeast1-b", "asia-northeast1-a";
"asia-south1" = "asia-south1-b", "asia-south1-a", "asia-south1-c";
"asia-southeast1" = "asia-southeast1-b", "asia-southeast1-a";
"australia-southeast1" = "australia-southeast1-a", "australia-southeast1-c", "australia-southeast1-b";
"europe-west1" = "europe-west1-b", "europe-west1-c", "europe-west1-d";
"europe-west2" = "europe-west2-a", "europe-west2-c", "europe-west2-b";
"europe-west3" = "europe-west3-b", "europe-west3-c", "europe-west3-a";
"southamerica-east1" = "southamerica-east1-b", "southamerica-east1-a", "southamerica-east1-c";
"us-central1" = "us-central1-f", "us-central1-b", "us-central1-c", "us-central1-a";
"us-east1" = "us-east1-d", "us-east1-b", "us-east1-c";
"us-east4" = "us-east4-a", "us-east4-b", "us-east4-c";
"us-west1" = "us-west1-c", "us-west1-a", "us-west1-b";
}
$title = "`nGoogle Cloud Platform Locations"

$message = "`nPlease select a GCP region for the PoC setup:"

Write-Host

$options = [System.Management.Automation.Host.ChoiceDescription[]]($GCPLocation)

$result = $host.ui.PromptForChoice($title, $message, $options, 0) 

$Location = $GCPLocation.Get($result)
write-host

Write-Host "`nThe selected region is $Location" -Foreground "Yellow"

$title = "Google Cloud Platform Zone"
$message = "`nPlease select a GCP zone for the selected region:"

Write-Host

$options = [System.Management.Automation.Host.ChoiceDescription[]]($GCPLocationToZones[$Location])

$result = $host.ui.PromptForChoice($title, $message, $options, 0) 

$Zone = $GCPLocationToZones[$Location].Get($result)
write-host

Write-Host "`nThe selected Zone is $Zone" -Foreground "Yellow"


$DeploymentName = Read-Host "`nEnter a name for the PoC Deployment to be created (Lowercase, Minimum Length – 8 characters)"	
$IAMName = $DeploymentName

$UniquePrefix = $DeploymentName


Write-Host

Write-Host "`nPoC setup network parameters" -Foreground "Green"

$DCNATip = Read-Host "`nEnter On-Premises Datacenter outbound public NAT ip"

$OnPremSub = Read-Host "`nEnter the On-Premises subnet prefix (example 192.168.xxx.xxx/xx)"

$AddressPrefix = Read-Host "`nEnter the GCP VPC CIDR block according to RFC1918 (example 10.10.xxx.xxx/xx)"

############################ Subnet1Public ##############################

$SubnetPublic = "velos-public-subnet.$UniquePrefix"


$CIDRsplit = $AddressPrefix.Split(@('/'))[0]
$CIDRsplit = $CIDRsplit.Split(@('.'))
$CIDRsplit[2] = [int]::Parse($CIDRsplit[2]) + 1

$SubnetPublic = [string]::Join(".", $CIDRsplit) + "/24"

############################ Subnet2Private ##############################

$SubnetPrivate = "velos-private-subnet.$UniquePrefix"


$CIDRsplit = $AddressPrefix.Split(@('/'))[0]
$CIDRsplit = $CIDRsplit.Split(@('.'))
$CIDRsplit[2] = [int]::Parse($CIDRsplit[2]) + 2

$SubnetPrivate = [string]::Join(".", $CIDRsplit) + "/24"



write-host "`nSummary: please verify the parameters entered for the PoC GCP setup:" -Foreground "Green"

write-host "`nPoC Deployment name"  -nonewline; write-host " - $DeploymentName" -Foreground "Yellow"

write-host "`nGCP Region" -nonewline; write-host " - $Location" -Foreground "Yellow"

write-host "`nGCP Region's Zone" -nonewline; write-host " - $Zone" -Foreground "Yellow"

write-host "`nDatacenter outbound public NAT ip" -nonewline; write-host " - $DCNATip" -Foreground "Yellow"

write-host "`nGCP PoC VPC CIDR block" -nonewline; write-host " - $AddressPrefix" -Foreground "Yellow"

write-host "`nPublic subnet will be set to" -nonewline; write-host " - $SubnetPublic" -Foreground "Yellow"

write-host "`nPrivate subnet will be set to" -nonewline; write-host " - $SubnetPrivate" -Foreground "Yellow"


$response = read-host "`nPress enter to continue or any other key (and then enter) to abort"
$aborted = [bool]$response
if ($aborted) {
	exit
}

Write-Host

Write-Host

$scriptpath = $MyInvocation.MyCommand.Path
$dir = Split-Path $scriptpath
Push-Location $dir

gcloud deployment-manager deployments create $DeploymentName --template ./templates/gcp_poc_setup_v1.0.jinja --properties "region:$Location,zone:$Zone,publicRange:$SubnetPublic,privateRange:$SubnetPrivate,sourceRange:$OnPremSub,sourceAddress:$DCNATip/32"

Write-Host
Write-Host "`nSetting IAM Service Account permissions" -foregroundcolor "Cyan"
Write-Host

$CEServiceAccount = "$IAMName-ce@$GCPProject.iam.gserviceaccount.com"
$MgmtServiceAccount = "$IAMName-mgmt@$GCPProject.iam.gserviceaccount.com"

gcloud projects add-iam-policy-binding $GCPProject --member serviceAccount:$CEServiceAccount --role projects/$GCPProject/roles/$IAMName"_ce_role"
gcloud projects add-iam-policy-binding $GCPProject --member serviceAccount:$MgmtServiceAccount --role projects/$GCPProject/roles/$IAMName"_mgmt_role"

(Get-Content ./templates/binding.yaml).replace('[MGMT_SA]', $MgmtServiceAccount) | Set-Content binding_m.yaml
gcloud iam service-accounts set-iam-policy $CEServiceAccount binding_m.yaml

gcloud iam service-accounts keys create $IAMName --iam-account=$serviceAccount.iam.gserviceaccount.com --key-file-type="json"

Write-Host "`nService Account Keys JSON file was created and downloaded to script's folder" -foregroundcolor "Cyan"

Write-Host "`nGCP Deployment of a Velostrata PoC setup was created successfully!" -foregroundcolor "Green"

Write-Host
